package data_manage

import (
	"github.com/rdlucklib/rdluck_tools/orm"
	"github.com/rdlucklib/rdluck_tools/paging"
)

type UpdatesStatisticalItem struct {
	Total  int
	Source int
}

func GetUpdatesStatistical(condition string, pars []interface{}) (list []*UpdatesStatisticalItem, err error) {
	o := orm.NewOrm()
	o.Using("data")
	sql := ` SELECT COUNT(1) AS total,source FROM edb_info
			WHERE 1=1 `
	sql += condition
	sql += ` GROUP BY source  `
	_, err = o.Raw(sql, pars).QueryRows(&list)
	return
}

type UpdatesStatisticalResp struct {
	ThsUpdate          int `description:"同花顺已更新"`
	ThsNotUpdate       int `description:"同花顺未更新"`
	WindUpdate         int `description:"wind已更新"`
	WindNotUpdate      int `description:"wind未更新"`
	PbUpdate           int `description:"彭博已更新"`
	PbNotUpdate        int `description:"彭博未更新"`
	LzUpdate           int `description:"隆众已更新"`
	LzNotUpdate        int `description:"隆众未更新"`
	SmmUpdate          int `description:"Smm已更新"`
	SmmNotUpdate       int `description:"Smm未更新"`
	MysteelUpdate      int `description:"钢联已更新"`
	MysteelNotUpdate   int `description:"钢联未更新"`
	CalculateUpdate    int `description:"计算指标已更新"`
	CalculateNotUpdate int `description:"计算指标未更新"`
	ManualUpdate       int `description:"手工指标已更新"`
	ManualNotUpdate    int `description:"手工指标未更新"`
}

/*
DATA_SOURCE_THS                 = iota + 1 //同花顺
	DATA_SOURCE_WIND                           //wind
	DATA_SOURCE_PB                             //彭博
	DATA_SOURCE_CALCULATE                      //指标运算
	DATA_SOURCE_CALCULATE_LJZZY                //累计值转月
	DATA_SOURCE_CALCULATE_TBZ                  //同比值
	DATA_SOURCE_CALCULATE_TCZ                  //同差值
	DATA_SOURCE_CALCULATE_NSZYDPJJS            //N数值移动平均计算
	DATA_SOURCE_MANUAL                         //手工指标
	DATA_SOURCE_LZ                             //隆众
	DATA_SOURCE_YS                             //有色
	DATA_SOURCE_CALCULATE_HBZ                  //环比值->12
	DATA_SOURCE_CALCULATE_HCZ                  //环差值->13
	DATA_SOURCE_CALCULATE_BP                   //变频->14
	DATA_SOURCE_GL                             //钢联->15
*/

func GetUpdatesList(condition string, pars []interface{}, startSize, pageSize int) (list []*EdbInfoView, err error) {
	o := orm.NewOrm()
	o.Using("data")
	sql := ` SELECT * FROM edb_info
			WHERE 1=1 `
	sql += condition
	sql += ` LIMIT ?,? `
	_, err = o.Raw(sql, pars, startSize, pageSize).QueryRows(&list)
	return
}

func GetUpdatesListTotal(condition string, pars []interface{}) (count int, err error) {
	o := orm.NewOrm()
	o.Using("data")
	sql := ` SELECT COUNT(1) AS count FROM edb_info
			WHERE 1=1 `
	sql += condition
	err = o.Raw(sql, pars).QueryRow(&count)
	return
}

type UpdatesListResp struct {
	List   []*EdbInfoView
	Total  int
	Paging *paging.PagingItem
}
