package data_manage

import (
	"github.com/rdlucklib/rdluck_tools/orm"
	"time"
)

type EdbdataClassify struct {
	ClassifyId   int       `orm:"column(classify_id);pk" description:"分类id"`
	ClassifyName string    `description:"分类名称"`
	ParentId     int       `description:"父级id"`
	CreateTime   time.Time `description:"创建时间"`
	Sort         int       `description:"排序"`
	IsShow       int       `description:"是否显示"`
}

type EdbdataClassifyList struct {
	ClassifyId   int       `description:"分类id"`
	ClassifyName string    `description:"分类名称"`
	ParentId     int       `description:"父级id"`
	CreateTime   time.Time `description:"创建时间"`
	Sort         int       `description:"排序"`
	IsShow       int       `description:"是否显示"`
	Total        int       `description:"指标总数"`
}

func AddEdbdataClassify(item *EdbdataClassify) (lastId int64, err error) {
	o := orm.NewOrm()
	o.Using("edb")
	lastId, err = o.Insert(item)
	return
}

type ManualClassifyList struct {
	ClassifyId   int
	ClassifyName string
	ParentId     int
	CheckList    []int
	Child        []*ManualClassifyList
}

func GetManualClassifyAll(adminId int) (items []*ManualClassifyList, err error) {
	var newItems []*ManualClassifyList
	o := orm.NewOrm()
	o.Using("edb")
	sql := ` SELECT classify_id,classify_name,parent_id FROM edbdata_classify WHERE parent_id=0 AND is_show=1 ORDER BY sort ASC `
	_, err = o.Raw(sql).QueryRows(&newItems)
	if err != nil {
		return
	}
	classifyLen := len(newItems)
	for i := 0; i < classifyLen; i++ {
		var childItems []*ManualClassifyList
		parentId := newItems[i].ClassifyId
		childSql := ``
		childSql = "SELECT classify_id,classify_name,parent_id FROM edbdata_classify WHERE parent_id=? AND is_show=1 ORDER BY sort ASC "
		_, err = o.Raw(childSql, parentId).QueryRows(&childItems)
		if err != nil {
			return
		}
		checkList := make([]int, 0)
		for _, v := range childItems {
			if v.ParentId > 0 {
				count, err := GetManualUserClassifyCount(adminId, v.ClassifyId)
				if err != nil {
					return items, err
				}
				if count > 0 {
					checkList = append(checkList, v.ClassifyId)
				}
			}
		}
		newItems[i].CheckList = checkList

		newItems[i].Child = childItems
	}

	for _, v := range newItems {
		if v.ParentId > 0 {
			count, err := GetManualUserClassifyCount(adminId, v.ClassifyId)
			if err != nil {
				return items, err
			}
			if count > 0 {
				v.CheckList = append(v.CheckList, v.ClassifyId)
			}
		}
		childLen := len(v.Child)
		if childLen > 0 {
			items = append(items, v)
		}
	}
	return
}

type AddManualClassifyReq struct {
	ClassifyName string `description:"分类名称"`
	ParentId     int    `description:"父级id,第一级传0"`
}

func GetManualClassifyCount(classifyName string, parentId int) (count int, err error) {
	o := orm.NewOrm()
	o.Using("edb")
	sql := `SELECT COUNT(1) AS count FROM edbdata_classify WHERE parent_id=? AND classify_name=? `
	err = o.Raw(sql, parentId, classifyName).QueryRow(&count)
	return
}

func GetManualClassifyByClassifyName(classifyName string) (item *EdbdataClassify, err error) {
	o := orm.NewOrm()
	o.Using("edb")
	sql := `SELECT * FROM edbdata_classify WHERE classify_name=? `
	err = o.Raw(sql, classifyName).QueryRow(&item)
	return
}

func ModifyManualClassifyIsShow(isShow, classifyId, parentId int) (err error) {
	o := orm.NewOrm()
	o.Using("edb")
	sql := `UPDATE edbdata_classify SET is_show=?,parent_id=? WHERE classify_id=? `
	_, err = o.Raw(sql, isShow, parentId, classifyId).Exec()
	return
}

type EditManualClassifyReq struct {
	ClassifyName string `description:"分类名称"`
	ClassifyId   int    `orm:"column(classify_id);pk" description:"分类id"`
}

func GetManualClassifyById(classifyId int) (item *EdbdataClassify, err error) {
	o := orm.NewOrm()
	o.Using("edb")
	sql := ` SELECT * FROM edbdata_classify WHERE classify_id=? `
	err = o.Raw(sql, classifyId).QueryRow(&item)
	return
}

func ModifyManualClassifyName(classifyName string, classifyId int) (err error) {
	o := orm.NewOrm()
	o.Using("edb")
	sql := `UPDATE edbdata_classify SET classify_name=? WHERE classify_id=? `
	_, err = o.Raw(sql, classifyName, classifyId).Exec()
	return
}

func GetManualClassifyAllByCondition(condition string) (items []*EdbdataClassifyList, err error) {
	o := orm.NewOrm()
	o.Using("edb")
	sql := ` SELECT classify_id,classify_name,parent_id FROM edbdata_classify WHERE is_show=1 `
	if condition != "" {
		sql += condition
	}
	sql += `  ORDER BY sort ASC `
	_, err = o.Raw(sql).QueryRows(&items)
	return
}
