package data

import (
	"context"
	"fmt"
	"github.com/olivere/elastic/v7"
	"hongze/hongze_chart_lib/models/data_manage"
	"hongze/hongze_chart_lib/utils"
	"log"
	"os"
	"strconv"
	"strings"
)

const (
	ES_URL      = "http://es-cn-nif227b580019rgw6.public.elasticsearch.aliyuncs.com:9200" //<1>
	ES_USERNAME = "elastic"                                                               //<2>
	ES_PASSWORD = "hongze@2021"                                                           //<3>
)

func NewClient() (client *elastic.Client, err error) {
	errorlog := log.New(os.Stdout, "APP", log.LstdFlags)
	file := ""
	if utils.RunMode == "release" {
		file = `./rdlucklog/eslog.log`
	} else {
		file = `./rdlucklog/eslog.log`
	}
	logFile, _ := os.OpenFile(file, os.O_RDWR|os.O_CREATE|os.O_APPEND, 0766)
	client, err = elastic.NewClient(
		elastic.SetURL(ES_URL),
		elastic.SetBasicAuth(ES_USERNAME, ES_PASSWORD),
		elastic.SetTraceLog(log.New(logFile, "ES-TRACE: ", 0)),
		elastic.SetSniff(false), elastic.SetErrorLog(errorlog))
	return
}


//indexName:索引名称
//mappingJson:表结构
func EsCreateIndex(indexName, mappingJson string) (err error) {
	client, err := NewClient()
	if err != nil {
		return
	}
	//定义表结构
	exists, err := client.IndexExists(indexName).Do(context.Background()) //<5>
	if err != nil {
		return
	}
	if !exists {
		resp, err := client.CreateIndex(indexName).BodyJson(mappingJson).Do(context.Background())
		//BodyJson(bodyJson).Do(context.Background())
		if err != nil {
			fmt.Println("CreateIndex Err:" + err.Error())
			return err
		}
		fmt.Println(resp.Index, resp.ShardsAcknowledged, resp.Acknowledged)
	} else {
		fmt.Println(indexName + " 已存在")
	}
	return
}

func AddAllEdbInfo() {
	allList, err := data_manage.GetEdbInfoAllList()
	if err != nil {
		fmt.Println("GetArticleAll Err:", err.Error())
		return
	}
	indexName := "hz_data_lib_v1"
	for _, v := range allList {
		EsAddOrEditEdbInfo(indexName, strconv.Itoa(v.EdbInfoId), v)
		fmt.Println(v.EdbInfoId)
	}
}

//新增和修改数据
func EsAddOrEditEdbInfo(indexName, docId string, item *data_manage.EdbInfo) (err error) {
	defer func() {
		if err != nil {
			fmt.Println("EsAddOrEditEdbInfo Err:", err.Error())
		}
	}()
	client, err := NewClient()
	if err != nil {
		return
	}
	searchById, err := client.Get().Index(indexName).Id(docId).Do(context.Background())
	if err != nil && !strings.Contains(err.Error(), "404") {
		fmt.Println("Get Err" + err.Error())
		return
	}
	if searchById != nil && searchById.Found {
		resp, err := client.Update().Index(indexName).Id(docId).Doc(map[string]interface{}{
			"EdbInfoId":  item.EdbInfoId,
			"EdbCode":    item.EdbCode,
			"EdbName":    item.EdbName,
			"Frequency":  item.Frequency,
			"SourceName": item.SourceName,
			"UniqueCode": item.UniqueCode,
			"Unit":       item.Unit,
		}).Do(context.Background())
		if err != nil {
			return err
		}
		fmt.Println(resp.Status, resp.Result)
		if resp.Status == 0 {
			fmt.Println("修改成功")
		} else {
			fmt.Println("EditData", resp.Status, resp.Result)
		}
	} else {
		resp, err := client.Index().Index(indexName).Id(docId).BodyJson(item).Do(context.Background())
		if err != nil {
			fmt.Println("新增失败:", err.Error())
			return err
		}
		if resp.Status == 0 && resp.Result == "created" {
			fmt.Println("新增成功")
			err = nil
		} else {
			fmt.Println("AddData", resp.Status, resp.Result)
		}
	}
	return
}
